import { Breadcrumb, DsnComponents, Event, Integration } from '@sentry/types';
import { Session } from './session/Session';
import { ReplayPerformanceEntry } from './createPerformanceEntry';
import { IEventBuffer } from './eventBuffer';
import type { InstrumentationType, RecordingConfig, RecordingEvent, ReplayRequest, ReplaySpan, SentryReplayConfiguration, SentryReplayPluginOptions } from './types';
/**
 * Returns true to return control to calling function, otherwise continue with normal batching
 */
declare type AddUpdateCallback = () => boolean | void;
export declare class SentryReplay implements Integration {
    /**
     * @inheritDoc
     */
    static id: string;
    /**
     * @inheritDoc
     */
    name: string;
    eventBuffer: IEventBuffer;
    /**
     * Buffer of breadcrumbs to be uploaded
     */
    breadcrumbs: Breadcrumb[];
    /**
     * Buffer of replay spans to be uploaded
     */
    replaySpans: ReplaySpan[];
    /**
     * List of PerformanceEntry from PerformanceObserver
     */
    performanceEvents: PerformanceEntry[];
    /**
     * Options to pass to `rrweb.record()`
     */
    readonly recordingOptions: RecordingConfig;
    readonly options: SentryReplayPluginOptions;
    /**
     * setTimeout id used for debouncing sending rrweb attachments
     */
    private timeout;
    /**
     * The timestamp of the first event since the last flush. This is used to
     * determine if the maximum allowed time has passed before events should be
     * flushed again.
     */
    private initialEventTimestampSinceFlush;
    private performanceObserver;
    private retryCount;
    private retryInterval;
    /**
     * Flag to make sure we only create a replay event when
     * necessary (i.e. we only want to have a single replay
     * event per session and it should only be created
     * immediately before sending recording)
     */
    private needsCaptureReplay;
    /**
     * Captured state when integration is first initialized
     */
    private initialState;
    session: Session | undefined;
    static attachmentUrlFromDsn(dsn: DsnComponents, eventId: string): string;
    constructor({ flushMinDelay, flushMaxDelay, initialFlushDelay, stickySession, // TBD: Making this opt-in for now
    useCompression, captureOnlyOnError, replaysSamplingRate, recordingConfig: { maskAllInputs, blockClass, ignoreClass, maskTextClass, ...recordingOptions }, }?: SentryReplayConfiguration);
    /**
     * Because we create a transaction in `setupOnce`, we can potentially create a
     * transaction before some native SDK integrations have run and applied their
     * own global event processor. An example is:
     * https://github.com/getsentry/sentry-javascript/blob/b47ceafbdac7f8b99093ce6023726ad4687edc48/packages/browser/src/integrations/useragent.ts
     *
     * So we call `this.setup` in next event loop as a workaround to wait for
     * other global event processors to finish
     */
    setupOnce(): void;
    /**
     * Initializes the plugin.
     *
     * Creates or loads a session, attaches listeners to varying events (DOM, PerformanceObserver, Recording, Sentry SDK, etc)
     */
    setup(): void;
    /**
     * We want to batch uploads of replay events. Save events only if
     * `<flushMinDelay>` milliseconds have elapsed since the last event
     * *OR* if `<flushMaxDelay>` milliseconds have elapsed.
     *
     * Accepts a callback to perform side-effects and returns true to stop batch
     * processing and hand back control to caller.
     */
    addUpdate(cb?: AddUpdateCallback): void;
    /**
     * Currently, this needs to be manually called (e.g. for tests). Sentry SDK does not support a teardown
     */
    destroy(): void;
    clearSession(): void;
    /**
     * Loads a session from storage, or creates a new one if it does not exist or
     * is expired.
     */
    loadSession({ expiry }: {
        expiry: number;
    }): void;
    /**
     * Adds listeners to record events for the replay
     */
    addListeners(): void;
    /**
     * Cleans up listeners that were created in `addListeners`
     */
    removeListeners(): void;
    /**
     * Core Sentry SDK global event handler. Attaches `replayId` to all [non-replay]
     * events as a tag. Also handles the case where we only want to capture a reply
     * when an error occurs.
     **/
    handleGlobalEvent: (event: Event) => Event;
    /**
     * Handler for recording events.
     *
     * Adds to event buffer, and has varying flushing behaviors if the event was a checkout.
     */
    handleRecordingEmit: (event: RecordingEvent, isCheckout?: boolean) => void;
    /**
     * Handle when visibility of the page content changes. Opening a new tab will
     * cause the state to change to hidden because of content of current page will
     * be hidden. Likewise, moving a different window to cover the contents of the
     * page will also trigger a change to a hidden state.
     */
    handleVisibilityChange: () => void;
    /**
     * Handle when page is blurred
     */
    handleWindowBlur: () => void;
    /**
     * Handle when page is focused
     */
    handleWindowFocus: () => void;
    /**
     * Handle when page is closed
     */
    handleWindowUnload: () => void;
    /**
     * Handler for Sentry Core SDK events.
     *
     * Transforms core SDK events into replay events.
     *
     */
    handleCoreListener: (type: InstrumentationType) => (handlerData: any) => void;
    /**
     * Keep a list of performance entries that will be sent with a replay
     */
    handlePerformanceObserver: (list: PerformanceObserverEntryList) => void;
    /**
     * Tasks to run when we consider a page to be hidden (via blurring and/or visibility)
     */
    doChangeToBackgroundTasks(breadcrumb?: Breadcrumb): void;
    /**
     * Tasks to run when we consider a page to be visible (via focus and/or visibility)
     */
    doChangeToForegroundTasks(breadcrumb?: Breadcrumb): void;
    /**
     * Trigger rrweb to take a full snapshot which will cause this plugin to
     * create a new Replay event.
     */
    triggerFullSnapshot(): void;
    /**
     * Updates the session's last activity timestamp
     */
    updateLastActivity(lastActivity?: number): void;
    /**
     * Helper to create (and buffer) a replay breadcrumb from a core SDK breadcrumb
     */
    createCustomBreadcrumb(breadcrumb: Breadcrumb): void;
    /**
     * Create a span for each performance entry. The parent transaction is `this.replayEvent`.
     */
    createPerformanceSpans(entries: ReplayPerformanceEntry[]): void;
    /**
     * Observed performance events are added to `this.performanceEvents`. These
     * are included in the replay event before it is finished and sent to Sentry.
     */
    addPerformanceEntries(): void;
    /**
     *
     *
     * Returns true if session is not expired, false otherwise.
     */
    checkAndHandleExpiredSession(expiry?: number): boolean;
    /**
     * Only flush if `captureOnlyOnError` is false.
     */
    conditionalFlush(lastActivity?: number): Promise<void>;
    /**
     * Flushes replay event buffer to Sentry.
     *
     * Performance events are only added right before flushing - this is probably
     * due to the buffered performance observer events.
     */
    flushUpdate(lastActivity?: number): Promise<void>;
    /**
     * Send replay attachment using either `sendBeacon()` or `fetch()`
     */
    sendReplayRequest({ endpoint, events }: ReplayRequest): Promise<void>;
    resetRetries(): void;
    /**
     * Finalize and send the current replay event to Sentry
     */
    sendReplay(eventId: string, events: Uint8Array | string): Promise<boolean>;
}
export {};
